# -*- coding: utf-8 -*-

from collections import Counter

from prosegrinder import Dictionary, Paragraph

dictionary = Dictionary()
text = """For easy readability, limit your choice of font to either Courier or Times New Roman. Courier (my strong preference) is a monospaced font, which means that every character is exactly as wide as every other. It's easier for an editor to detect spelling errors in a monospaced font than in a proportional font like Times New Roman (in which the "i" uses less horizontal space than the "m" does). With a monospaced font, there will also be fewer characters on each line, which can make your manuscript easier to scan. Still, many writers have come to prefer Times New Roman, and either is usually acceptable. (If in doubt, consult your intended market's submission guidelines.) Set your font size to 12 points."""
paragraph = Paragraph(text, dictionary)

WORD_CHARACTER_COUNT = 563
PHONE_COUNT = 55 + 72 + 118 + 73 + 63 + 47 + 21
SYLLABLE_COUNT = 195
WORD_COUNT = 121
COMPLEX_WORD_COUNT = 22
LONG_WORD_COUNT = 22
UNIQUE_WORD_COUNT = 83
POV_WORD_COUNT = 6
FIRST_PERSON_INDICATOR_COUNT = 1
SECOND_PERSON_INDICATOR_COUNT = 4
THIRD_PERSON_INDICATOR_COUNT = 1
SENTENCE_COUNT = 7


def test_characters():
    assert(WORD_CHARACTER_COUNT == paragraph.word_character_count)


def test_phones():
    assert(PHONE_COUNT == paragraph.phone_count)


def test_syllables():
    assert(SYLLABLE_COUNT == paragraph.syllable_count)


def test_words():
    assert(WORD_COUNT == paragraph.word_count)
    assert(COMPLEX_WORD_COUNT == paragraph.complex_word_count)
    assert(LONG_WORD_COUNT == paragraph.long_word_count)
    assert(UNIQUE_WORD_COUNT == paragraph.unique_word_count)


def test_pov():
    assert(POV_WORD_COUNT == paragraph.pov_word_count)
    assert(FIRST_PERSON_INDICATOR_COUNT == paragraph.first_person_word_count)
    assert(SECOND_PERSON_INDICATOR_COUNT == paragraph.second_person_word_count)
    assert(THIRD_PERSON_INDICATOR_COUNT == paragraph.third_person_word_count)


def test_sentences():
    assert(SENTENCE_COUNT == paragraph.sentence_count)


def test_dialogue_narrative():
    assert(WORD_CHARACTER_COUNT == paragraph.dialogue.word_character_count +
           paragraph.narrative.word_character_count)
    assert(SYLLABLE_COUNT == paragraph.dialogue.syllable_count +
           paragraph.narrative.syllable_count)
    assert(WORD_COUNT == paragraph.dialogue.word_count +
           paragraph.narrative.word_count)
    assert(LONG_WORD_COUNT == paragraph.dialogue.long_word_count +
           paragraph.narrative.long_word_count)
    assert(COMPLEX_WORD_COUNT == paragraph.dialogue.complex_word_count +
           paragraph.narrative.complex_word_count)
    combined_word_frequency = Counter()
    combined_word_frequency.update(paragraph.dialogue.word_frequency)
    combined_word_frequency.update(paragraph.narrative.word_frequency)
    assert(UNIQUE_WORD_COUNT == len(combined_word_frequency))
    assert(POV_WORD_COUNT == paragraph.dialogue.pov_word_count +
           paragraph.narrative.pov_word_count)
    assert(FIRST_PERSON_INDICATOR_COUNT == paragraph.dialogue.first_person_word_count +
           paragraph.narrative.first_person_word_count)
    assert(SECOND_PERSON_INDICATOR_COUNT == paragraph.dialogue.second_person_word_count +
           paragraph.narrative.second_person_word_count)
    assert(THIRD_PERSON_INDICATOR_COUNT == paragraph.dialogue.third_person_word_count +
           paragraph.narrative.third_person_word_count)



# -*- coding: utf-8 -*-

import re
from collections import Counter

import pointofview


class Word():

    """A Word, the base unit for measuring fiction prose."""

    MIN_SYLLABLES_COMPLEX_WORD = 3
    MIN_CHARS_LONG_WORD = 7
    # See: https://en.wikipedia.org/wiki/List_of_Unicode_characters
    RE_WORD = re.compile(
        "[\\w’'\u0391-\u03ce\u0400-\u0481\u048a-\u04ff]+")

    def __init__(self, text, phones, normalized_phones, syllable_count, is_dictionary_word, is_numeric):
        """Assumes text is a single word, normalized."""
        self._text = text
        self._phones = phones
        self._normalized_phones = normalized_phones
        self._phone_count = len(self._normalized_phones)
        self._phone_frequency = dict(Counter(self._normalized_phones))
        self._syllable_count = syllable_count
        self._is_dictionary_word = is_dictionary_word
        self._is_numeric = is_numeric
        self._character_count = len(self._text)
        self._pov = pointofview.get_word_pov(self._text)

    def __str__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    def __hash__(self):
        return hash(self._text)

    @property
    def text(self):
        """Return the text initially used to create the Word."""
        return self._text

    @property
    def phones(self):
        """Return the Word's phones with syllable marks."""
        return self._phones

    @property
    def normalized_phones(self):
        """Return the Word's phones without syllable marks."""
        return self._normalized_phones

    @property
    def phone_frequency(self):
        """Return the frequency of phones found in Word."""
        return self._phone_frequency

    @property
    def phone_count(self):
        """Return the number of phones found in Word."""
        return self._phone_count

    @property
    def syllable_count(self):
        """Return the number of syllables in Word."""
        return self._syllable_count

    @property
    def is_dictionary_word(self):
        """Whether or not the Word was found in the Dictionary."""
        return self._is_dictionary_word

    @property
    def is_numeric(self):
        """Returns whether or not the Word is numeric per the Dictionary."""
        return self._is_numeric

    @property
    def character_count(self):
        """Returns the number of characters in the Word."""
        return self._character_count

    @property
    def is_complex_word(self):
        """Returns whether or not the Word is considered complex."""
        return self.syllable_count >= Word.MIN_SYLLABLES_COMPLEX_WORD

    @property
    def is_long_word(self):
        """Returns whether or not the Word is considered long."""
        return self.character_count >= Word.MIN_CHARS_LONG_WORD

    @property
    def is_first_person_word(self):
        """Returns whether or not the Word inidicates first-peron point of view."""
        return self._pov == 'first'

    @property
    def is_second_person_word(self):
        """Returns whether or not the Word inidicates second-peron point of view."""
        return self._pov == 'second'

    @property
    def is_third_person_word(self):
        """Returns whether or not the Word inidicates third-peron point of view."""
        return self._pov == 'third'

    @property
    def is_pov_word(self):
        """Returns whether or not the Word inidicates any point of view."""
        return (self.is_first_person_word or
                self.is_second_person_word or
                self.is_third_person_word)

    @property
    def pov(self):
        return self._pov



# -*- coding: utf-8 -*-

from collections import Counter

import narrative

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment
from prosegrinder.fragment_container import FragmentContainer
from prosegrinder.paragraph import Paragraph
from prosegrinder.readability_scores import ReadabilityScores


class Prose():

    def __init__(self, text, dictionary=Dictionary()):
        self._text = text
        self._dictionary = dictionary
        self._paragraphs = Paragraph.parse_paragraphs(
            self._text, self._dictionary)
        self._word_character_count = sum(
            [paragraph.word_character_count for paragraph in self._paragraphs])
        self._syllable_count = sum(
            [paragraph.syllable_count for paragraph in self._paragraphs])
        self._word_count = sum(
            [paragraph.word_count for paragraph in self._paragraphs])
        self._complex_word_count = sum(
            [paragraph.complex_word_count for paragraph in self._paragraphs])
        self._long_word_count = sum(
            [paragraph.long_word_count for paragraph in self._paragraphs])
        self._pov_word_count = sum(
            [paragraph.pov_word_count for paragraph in self._paragraphs])
        self._first_person_word_count = sum(
            [paragraph.first_person_word_count for paragraph in self._paragraphs])
        self._second_person_word_count = sum(
            [paragraph.second_person_word_count for paragraph in self._paragraphs])
        self._third_person_word_count = sum(
            [paragraph.third_person_word_count for paragraph in self._paragraphs])
        wf = Counter()
        pf = Counter()
        pc = 0
        for paragraph in self._paragraphs:
            wf.update(paragraph.word_frequency)
            pf.update(paragraph.phone_frequency)
            pc += paragraph.phone_count
        self._word_frequency = dict(wf)
        self._phone_frequency = dict(pf)
        self._phone_count = pc
        self._sentence_count = sum(
            [paragraph.sentence_count for paragraph in self._paragraphs])
        self._paragraph_count = len(self._paragraphs)
        self._readability_scores = ReadabilityScores(
            self._word_character_count, self._syllable_count, self._word_count,
            self._complex_word_count, self._long_word_count, self._sentence_count)
        n = narrative.split(self._text)
        self._dialogue = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in n['dialogue']])
        self._narrative = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in n['narrative']])
        self._pov = self._narrative.pov

    def __str__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        return self._text == other._text

    def __hash__(self):
        return hash(self._text)

    @property
    def dictionary(self):
        return self._dictionary

    @property
    def phone_frequency(self):
        return self._phone_frequency

    @property
    def phone_count(self):
        return self._phone_count

    @property
    def word_character_count(self):
        return self._word_character_count

    @property
    def syllable_count(self):
        return self._syllable_count

    @property
    def word_count(self):
        return self._word_count

    @property
    def complex_word_count(self):
        return self._complex_word_count

    @property
    def long_word_count(self):
        return self._long_word_count

    @property
    def unique_word_count(self):
        return len(self._word_frequency)

    @property
    def pov_word_count(self):
        return self._pov_word_count

    @property
    def first_person_word_count(self):
        return self._first_person_word_count

    @property
    def second_person_word_count(self):
        return self._second_person_word_count

    @property
    def third_person_word_count(self):
        return self._third_person_word_count

    @property
    def sentence_count(self):
        return self._sentence_count

    @property
    def paragrah_count(self):
        return self._paragraph_count

    @property
    def readability_scores(self):
        return self._readability_scores

    @property
    def dialogue(self):
        return self._dialogue

    @property
    def narrative(self):
        return self._narrative

    @property
    def pov(self):
        return self._pov

    @property
    def text(self):
        return self._text



# -*- coding: utf-8 -*-

from prosegrinder import Word

def test_equality():
    w1 = Word(
        'frowning',
        ["F", "R", "AW1", "N", "IH0", "NG"],
        ["F", "R", "AW", "N", "IH", "NG"],
        2,
        True,
        False)
    w2 = Word(
        'frowning',
        ["F", "R", "AW1", "N", "IH0", "NG"],
        ["F", "R", "AW", "N", "IH", "NG"],
        2,
        True,
        False)
    w3 = Word(
        'cafe',
        ["K", "AH0", "F", "EY1"],
        ["K", "AH", "F", "EY"],
        2,
        True,
        False)
    assert(w1 == w2)
    assert(w1 != w3)
    assert(w2 != w3)



# -*- coding: utf-8 -*-

import re
from collections import Counter

import pointofview

from prosegrinder.dictionary import Dictionary
from prosegrinder.word import Word


class Fragment():

    def __init__(self, text, dictionary=Dictionary()):
        self._text = text
        self._dictionary = dictionary
        self._normalized_sentence = dictionary.normalize_text(text)
        self._words = [self._dictionary.get_word(word) for word in re.findall(
            Word.RE_WORD, self._normalized_sentence)]
        self._word_count = len(self._words)
        self._word_character_count = sum(
            [word.character_count for word in self._words])
        pf = Counter()
        pc = 0
        for word in self._words:
            pf.update(word.phone_frequency)
            pc += word.phone_count
        self._phone_fequency = pf
        self._phone_count = pc
        self._syllable_count = sum(
            [word.syllable_count for word in self._words])
        self._complex_word_count = sum(
            [word.is_complex_word for word in self._words])
        self._long_word_count = sum(
            [word.is_long_word for word in self._words])
        self._pov_word_count = sum([word.is_pov_word for word in self._words])
        self._first_person_word_count = sum(
            [word.is_first_person_word for word in self._words])
        self._second_person_word_count = sum(
            [word.is_second_person_word for word in self._words])
        self._third_person_word_count = sum(
            [word.is_third_person_word for word in self._words])
        self._word_frequency = dict(Counter(self._words))
        self._pov = pointofview.NONE
        if self._first_person_word_count > 0:
            self._pov = pointofview.FIRST
        elif self._second_person_word_count > 0:
            self._pov = pointofview.SECOND
        elif self._third_person_word_count > 0:
            self._pov = pointofview.THIRD

    def __str__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        return self._text == other._text

    def __hash__(self):
        return hash(self._text)

    @property
    def dictionary(self):
        return self._dictionary

    @property
    def word_count(self):
        return self._word_count

    @property
    def word_character_count(self):
        return self._word_character_count

    @property
    def phone_frequency(self):
        return self._phone_fequency

    @property
    def phone_count(self):
        return self._phone_count

    @property
    def syllable_count(self):
        return self._syllable_count

    @property
    def complex_word_count(self):
        return self._complex_word_count

    @property
    def long_word_count(self):
        return self._long_word_count

    @property
    def unique_words(self):
        return self._word_frequency.keys()

    @property
    def word_frequency(self):
        return self._word_frequency

    @property
    def pov_word_count(self):
        return self._pov_word_count

    @property
    def text(self):
        return self._text

    @property
    def frequency(self, word_string):
        return self._word_frequency[word_string]

    @property
    def first_person_word_count(self):
        return self._first_person_word_count

    @property
    def second_person_word_count(self):
        return self._second_person_word_count

    @property
    def third_person_word_count(self):
        return self._third_person_word_count

    @property
    def words(self):
        return self._words

    @property
    def pov(self):
        return self._pov



# -*- coding: utf-8 -*-

import re

import narrative

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment
from prosegrinder.fragment_container import FragmentContainer
from prosegrinder.sentence import Sentence


class Paragraph(FragmentContainer):

    RE_PARAGRAPH = re.compile(".*(?=\\n|$)")

    def __init__(self, text, dictionary=Dictionary()):
        self._text = text
        self._dictionary = dictionary
        self._sentences = Sentence.parse_sentences(
            self._text, self._dictionary)
        n = narrative.split(self._text)
        self._dialogue = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in n['dialogue']])
        self._narrative = FragmentContainer(
            [Fragment(fragment_text) for fragment_text in n['narrative']])
        self._pov = self._narrative.pov
        super().__init__(self._sentences, self._dictionary)

    def __eq__(self, other):
        return self._text == other._text

    @property
    def sentence_count(self):
        return self.fragment_count

    @property
    def dialogue(self):
        return self._dialogue

    @property
    def narrative(self):
        return self._narrative

    @property
    def pov(self):
        return self._pov

    @staticmethod
    def parse_paragraphs(text, dictionary=Dictionary()):
        return [Paragraph(paragraph, dictionary) for paragraph in re.findall(
            Paragraph.RE_PARAGRAPH, text)]



# -*- coding: utf-8 -*-

from collections import Counter

import pointofview

from prosegrinder.dictionary import Dictionary


class FragmentContainer():

    def __init__(self, fragments, dictionary=Dictionary()):
        self._dictionary = dictionary
        self._fragments = fragments or []
        self._word_count = sum(
            [fragment.word_count for fragment in self._fragments])
        self._word_character_count = sum(
            [fragment.word_character_count for fragment in self._fragments])
        self._syllable_count = sum(
            [fragment.syllable_count for fragment in self._fragments])
        self._complex_word_count = sum(
            [fragment.complex_word_count for fragment in self._fragments])
        self._long_word_count = sum(
            [fragment.long_word_count for fragment in self._fragments])
        self._pov_word_count = sum(
            [fragment.pov_word_count for fragment in self._fragments])
        self._first_person_word_count = sum(
            [fragment.first_person_word_count for fragment in self._fragments])
        self._second_person_word_count = sum(
            [fragment.second_person_word_count for fragment in self._fragments])
        self._third_person_word_count = sum(
            [fragment.third_person_word_count for fragment in self._fragments])
        wf = Counter()
        pf = Counter()
        pc = 0
        for fragment in self._fragments:
            wf.update(fragment.words)
            pf.update(fragment.phone_frequency)
            pc += fragment.phone_count
        self._word_frequency = dict(wf)
        self._phone_frequency = dict(pf)
        self._phone_count = pc
        self._pov = pointofview.NONE
        if (self._first_person_word_count > 0):
            self._pov = pointofview.FIRST
        elif (self._second_person_word_count > 0):
            self._pov = pointofview.SECOND
        elif (self._third_person_word_count > 0):
            self._pov = pointofview.THIRD

    def __str__(self):
        return str(self.__dict__)

    def __eq__(self, other):
        return self._fragments == other._fragments

    def __hash__(self):
        return hash(self._fragments)

    @property
    def dictionary(self):
        return self._dictionary

    @property
    def phone_frequency(self):
        return self._phone_frequency

    @property
    def phone_count(self):
        return self._phone_count

    @property
    def word_count(self):
        return self._word_count

    @property
    def word_character_count(self):
        return self._word_character_count

    @property
    def syllable_count(self):
        return self._syllable_count

    @property
    def complex_word_count(self):
        return self._complex_word_count

    @property
    def long_word_count(self):
        return self._long_word_count

    @property
    def unique_word_count(self):
        return len(self.unique_words)

    @property
    def unique_words(self):
        return self._word_frequency.keys()

    @property
    def word_frequency(self):
        return self._word_frequency

    @property
    def pov_word_count(self):
        return self._pov_word_count

    @property
    def fragments(self):
        return self._fragments

    @property
    def frequency(self, word_string):
        return self._word_frequency[self._dictionary.get_word(word_string)]

    @property
    def fragment_count(self):
        return len(self._fragments)

    @property
    def first_person_word_count(self):
        return self._first_person_word_count

    @property
    def second_person_word_count(self):
        return self._second_person_word_count

    @property
    def third_person_word_count(self):
        return self._third_person_word_count

    @property
    def pov(self):
        return self._pov



# -*- coding: utf-8 -*-

from math import sqrt


class ReadabilityScores():

    NDIGITS = 3  # Default for round(number,[ndigits])

    def __init__(self, word_character_count, syllable_count, word_count,
                 complex_word_count, long_word_count, sentence_count, ndigits=NDIGITS):
        self._word_character_count = word_character_count
        self._syllable_count = syllable_count
        self._word_count = word_count
        self._complex_word_count = complex_word_count
        self._long_word_count = long_word_count
        self._sentence_count = sentence_count
        self._ndigits = ndigits

        self._automated_readability_index = self.calculate_automated_readability_index(
            self._word_character_count, self._word_count, self._sentence_count, self._ndigits
        )
        self._coleman_liau_index = self.calculate_coleman_liau_index(
            self._word_character_count, self._word_count, self._sentence_count, self._ndigits
        )
        self._flesch_kincaid_grade_level = self.calculate_flesch_kincaid_grade_level(
            self._syllable_count, self._word_count, self._sentence_count, self._ndigits
        )
        self._flesch_reading_ease = self.calculate_flesch_reading_ease(
            self._syllable_count, self._word_count, self._sentence_count, self._ndigits
        )
        self._gunning_fog_index = self.calculate_gunning_fog_index(
            self._word_count, self._complex_word_count, self._sentence_count, self._ndigits
        )
        self._linsear_write = self.calculate_linsear_write(
            self._word_count, self._complex_word_count, self._sentence_count, self._ndigits
        )
        self._lix = self.calculate_lix(
            self._word_count, self._long_word_count, self._sentence_count, self._ndigits
        )
        self._rix = self.calculate_rix(
            self._long_word_count, self._sentence_count, self._ndigits
        )
        self._smog = self.calculate_smog(
            self._complex_word_count, self._sentence_count, self._ndigits
        )

    @property
    def ndigits(self):
        return self._ndigits

    @property
    def automated_readability_index(self):
        return self._automated_readability_index

    @staticmethod
    def calculate_automated_readability_index(word_character_count, word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            avg_characters_per_word = word_character_count / word_count
            avg_words_per_sentence = word_count / word_character_count
            score = (4.71 * avg_characters_per_word) + \
                (0.5 * avg_words_per_sentence) - 21.43
        return round(score, ndigits)

    @property
    def coleman_liau_index(self):
        return self._coleman_liau_index

    @staticmethod
    def calculate_coleman_liau_index(word_character_count, word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            avg_letters_per_word = word_character_count / word_count * 100.0
            avg_sentences_per_word = sentence_count / word_count * 100.0
            score = (0.0588 * avg_letters_per_word) - \
                (0.296 * avg_sentences_per_word) - 15.8
        return round(score, ndigits)

    @property
    def flesch_kincaid_grade_level(self):
        return self._flesch_kincaid_grade_level

    @staticmethod
    def calculate_flesch_kincaid_grade_level(syllable_count, word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            avg_sentence_length = word_count / sentence_count
            avg_syllables_per_word = syllable_count / word_count
            score = (0.39 * avg_sentence_length) + \
                (11.8 * avg_syllables_per_word) - 15.59
        return round(score, ndigits)

    @property
    def flesch_reading_ease(self):
        return self._flesch_reading_ease

    @staticmethod
    def calculate_flesch_reading_ease(syllable_count, word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            avg_sentence_length = word_count / sentence_count
            avg_syllables_per_word = syllable_count / word_count
            score = 206.835 - (1.015 * avg_sentence_length) - \
                (84.6 * avg_syllables_per_word)
        return round(score, ndigits)

    @property
    def gunning_fog_index(self):
        return self._gunning_fog_index

    @staticmethod
    def calculate_gunning_fog_index(word_count, complex_word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            avg_sentence_length = word_count / sentence_count
            pct_hard_words = complex_word_count / word_count * 100.0
            score = 0.4 * (avg_sentence_length + pct_hard_words)
        return round(score, ndigits)

    @property
    def linsear_write(self):
        return self._linsear_write

    @staticmethod
    def calculate_linsear_write(word_count, complex_word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        hard_word_count = complex_word_count
        easy_word_count = (word_count - complex_word_count)
        if (sentence_count > 0):
            r = (easy_word_count + (hard_word_count * 3)) / sentence_count
            if (r <= 20):
                r = r - 2
            score = r / 2
        return round(score, ndigits)

    @property
    def lix(self):
        return self._lix

    @staticmethod
    def calculate_lix(word_count, long_word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (word_count > 0):
            score = word_count / sentence_count + \
                (100 * long_word_count) / word_count
        return round(score, ndigits)

    @property
    def rix(self):
        return self._rix

    @staticmethod
    def calculate_rix(long_word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (sentence_count > 0):
            score = long_word_count / sentence_count
        return round(score, ndigits)

    @property
    def smog(self):
        return self._smog

    @staticmethod
    def calculate_smog(complex_word_count, sentence_count, ndigits=NDIGITS):
        score = 0.0
        if (sentence_count > 0):
            score = (1.0430 * sqrt(complex_word_count *
                                   (30 / sentence_count))) + 3.1291
        return round(score, ndigits)



# -*- coding: utf-8 -*-

from prosegrinder import Dictionary
from prosegrinder import Word

dictionary = Dictionary()

def test_syllable_count():
    assert(dictionary.syllable_count("frowning") == 2)
    assert(dictionary.syllable_count("zurkuhlen") == 3)
    assert(dictionary.syllable_count("cafe") == 2)
    assert(dictionary.syllable_count("20,012.12") == 7)
    assert(dictionary.syllable_count("1,904") == 4)
    assert(dictionary.syllable_count("0.2315") == 5)
    assert(dictionary.syllable_count("-503,012.12") == 9)

def test_get_word():
    frowning = Word(
        'frowning',
        ["F", "R", "AW1", "N", "IH0", "NG"],
        ["F", "R", "AW", "N", "IH", "NG"],
        2,
        True,
        False)
    assert(frowning == dictionary.get_word('frowning'))
    cafe = Word(
        'cafe',
        ["K", "AH0", "F", "EY1"],
        ["K", "AH", "F", "EY"],
        2,
        True,
        False)
    assert(cafe == dictionary.get_word('cafe'))
    zurkuhlen = Word(
        'zurkuhlen',
        ["Z", "ER0", "K", "Y", "UW1", "L", "AH0", "N"],
        ["Z", "ER", "K", "Y", "UW", "L", "AH", "N"],
        3,
        True,
        False)
    assert(zurkuhlen == dictionary.get_word('zurkuhlen'))
    num1904 = Word(
        '1,904',
        ["?"],
        ["?"],
        4,
        False,
        True)
    assert(num1904 == dictionary.get_word('1,904'))



# -*- coding: utf-8 -*-

from prosegrinder import ReadabilityScores


COMPLEX_WORD_COUNT = 202
LONG_WORD_COUNT = 275
SENTENCE_COUNT = 90
SYLLABLE_COUNT = 2287
WORD_COUNT = 1528
CHARACTER_COUNT = 7008

AUTOMATED_READABILITY_INDEX = 0.281
COLEMAN_LIAU_INDEX = 9.425
FLESCH_KINCAID_GRADE_LEVEL = 8.693
FLESCH_READING_EASE = 62.979
GUNNING_FOG_INDEX = 12.079
LINSEAR_WRITE = 10.733
LIX = 34.975
RIX = 3.056
SMOG = 11.688


def test_calculate_automated_readability_index():
    score = ReadabilityScores.calculate_automated_readability_index(
        CHARACTER_COUNT, WORD_COUNT, SENTENCE_COUNT)
    assert(AUTOMATED_READABILITY_INDEX == score)


def test_calculate_coleman_liau_index():
    score = ReadabilityScores.calculate_coleman_liau_index(
        CHARACTER_COUNT, WORD_COUNT, SENTENCE_COUNT)
    assert(COLEMAN_LIAU_INDEX == score)


def test_calculate_flesch_kincaid_grade_level():
    score = ReadabilityScores.calculate_flesch_kincaid_grade_level(
        SYLLABLE_COUNT, WORD_COUNT, SENTENCE_COUNT)
    assert(FLESCH_KINCAID_GRADE_LEVEL == score)


def test_calculate_flesch_reading_ease():
    score = ReadabilityScores.calculate_flesch_reading_ease(
        SYLLABLE_COUNT, WORD_COUNT, SENTENCE_COUNT)
    assert(FLESCH_READING_EASE == score)


def test_calculate_gunning_fog_index():
    score = ReadabilityScores.calculate_gunning_fog_index(
        WORD_COUNT, COMPLEX_WORD_COUNT, SENTENCE_COUNT)
    assert(GUNNING_FOG_INDEX == score)


def test_linsear_write():
    score = ReadabilityScores.calculate_linsear_write(
        WORD_COUNT, COMPLEX_WORD_COUNT,SENTENCE_COUNT)
    assert(LINSEAR_WRITE == score)

def test_calculate_lix():
    score = ReadabilityScores.calculate_lix(
        WORD_COUNT, LONG_WORD_COUNT, SENTENCE_COUNT)
    assert(LIX == score)


def test_calculate_rix():
    score = ReadabilityScores.calculate_rix(LONG_WORD_COUNT, SENTENCE_COUNT)
    assert(RIX == score)


def test_calculate_smog():
    score = ReadabilityScores.calculate_smog(
        COMPLEX_WORD_COUNT, SENTENCE_COUNT)
    assert(SMOG == score)



# -*- coding: utf-8 -*-

import re

import cmudict
import syllables

from prosegrinder.word import Word


class Dictionary():

    """A reference containing Words."""

    RE_NUMERIC = re.compile("^[+-]{0,1}\\d{1,3}(?:[,]\\d{3})*(?:[.]\\d*)*$")

    @staticmethod
    def normalize_text(text):
        return text.strip().lower()

    @staticmethod
    def normalize_phones(phones):
        return [re.sub("\d", "", phone) for phone in phones]

    @staticmethod
    def is_numeric(word):
        return re.match(Dictionary.RE_NUMERIC, word) is not None

    def __init__(self, cmudictdict=None):
        self._cmudictdict = cmudictdict or cmudict.dict()

    def phones(self, word):
        if word in self._cmudictdict:
            return self._cmudictdict[word][0]
        else:
            return ['?']

    def syllable_count(self, word):
        syllable_count = 0
        if word in self._cmudictdict:
            phones = self._cmudictdict[word][0]
            # There's a more Pythonic way of doing this.
            for phone in phones:
                syllable_count += len(re.sub("[^012]", "", phone))
        elif self.is_numeric(word):
            syllable_count = len(re.sub('[^\\d\\+-]', '', word))
        else:
            syllable_count = syllables.estimate(word)
        return syllable_count

    @property
    def cmudictdict(self):
        return self._cmudictdict

    def get_word(self, word):
        normalized_word = self.normalize_text(word)
        phones = self.phones(normalized_word)
        normalized_phones = self.normalize_phones(phones)
        syllable_count = self.syllable_count(normalized_word)
        is_dictionary_word = normalized_word in self._cmudictdict
        is_numeric = self.is_numeric(normalized_word)
        word = Word(normalized_word, phones, normalized_phones, syllable_count,
                    is_dictionary_word, is_numeric)
        return word



# -*- coding: utf-8 -*-

import pkg_resources

from prosegrinder.word import Word
from prosegrinder.dictionary import Dictionary
from prosegrinder.sentence import Sentence
from prosegrinder.paragraph import Paragraph
from prosegrinder.prose import Prose
from prosegrinder.readability_scores import ReadabilityScores


__version__ = pkg_resources.resource_string(
    'prosegrinder', 'VERSION').decode('utf-8').strip()



# -*- coding: utf-8 -*-

from os import path

from setuptools import setup

# Version
with open(path.join(path.dirname(__file__), 'prosegrinder', 'VERSION')) as version_file:
    VERSION = version_file.read().strip()
# Long Description
with open(path.join(path.dirname(__file__), 'README.rst')) as readme_file:
    LONG_DESCRIPTION = readme_file.read()

setup(
    name='prosegrinder',
    version=VERSION,
    description='A text analytics library for prose fiction.',
    long_description=LONG_DESCRIPTION,
    author='David L. Day',
    author_email='dday376@gmail.com',
    url='https://github.com/prosegrinder/python-prosegrinder',
    packages=[
        'prosegrinder'
    ],
    package_dir={'prosegrinder': 'prosegrinder'},
    package_data={
        '': ['LICENSE', '*.rst', 'MANIFEST.in'],
    },
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    install_requires=[
        'cmudict>=1.0.0',
        'narrative>=1.0.0',
        'pointofview>=1.0.0',
        'syllables>=1.0.0'
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-

import os
from collections import Counter

from prosegrinder import Dictionary, Prose

dictionary = Dictionary()
SHORTSTORY = os.path.join(os.path.dirname(
    __file__), 'resources', 'shortstory.txt')
text = open(SHORTSTORY).read()
prose = Prose(text)

WORD_CHARACTER_COUNT = 7008
PHONE_COUNT = 5747
SYLLABLE_COUNT = 2287
COMPLEX_WORD_COUNT = 202
LONG_WORD_COUNT = 275
SENTENCE_COUNT = 90
WORD_COUNT = 1528
UNIQUE_WORD_COUNT = 526
POV_WORD_COUNT = 113  # was 104
FIRST_PERSON_INDICATOR_COUNT = 8  # was 6
SECOND_PERSON_INDICATOR_COUNT = 74  # was 73
THIRD_PERSON_INDICATOR_COUNT = 31  # was 25
PARAGRAPH_COUNT = 77

AUTOMATED_READABILITY_INDEX = 0.281
COLEMAN_LIAU_INDEX = 9.425
FLESCH_KINCAID_GRADE_LEVEL = 8.693
FLESCH_READING_EASE = 62.979
GUNNING_FOG_INDEX = 12.079
LIX = 34.975
RIX = 3.056
SMOG = 11.688


def test_characters():
    assert(WORD_CHARACTER_COUNT == prose.word_character_count)


def test_phones():
    assert(PHONE_COUNT == prose.phone_count)


def test_syllables():
    assert(SYLLABLE_COUNT == prose.syllable_count)


def test_words():
    assert(WORD_COUNT == prose.word_count)
    assert(COMPLEX_WORD_COUNT == prose.complex_word_count)
    assert(LONG_WORD_COUNT == prose.long_word_count)
    assert(UNIQUE_WORD_COUNT == prose.unique_word_count)


def test_pov():
    assert(POV_WORD_COUNT == prose.pov_word_count)
    assert(FIRST_PERSON_INDICATOR_COUNT == prose.first_person_word_count)
    assert(SECOND_PERSON_INDICATOR_COUNT == prose.second_person_word_count)
    assert(THIRD_PERSON_INDICATOR_COUNT == prose.third_person_word_count)


def test_sentences():
    assert(SENTENCE_COUNT == prose.sentence_count)


def test_paragraphs():
    assert(PARAGRAPH_COUNT == prose.paragrah_count)


def test_dialogue_narrative():
    assert(WORD_CHARACTER_COUNT == prose.dialogue.word_character_count +
           prose.narrative.word_character_count)
    assert(SYLLABLE_COUNT == prose.dialogue.syllable_count +
           prose.narrative.syllable_count)
    assert(WORD_COUNT == prose.dialogue.word_count +
           prose.narrative.word_count)
    assert(LONG_WORD_COUNT == prose.dialogue.long_word_count +
           prose.narrative.long_word_count)
    assert(COMPLEX_WORD_COUNT == prose.dialogue.complex_word_count +
           prose.narrative.complex_word_count)
    combined_word_frequency = Counter()
    combined_word_frequency.update(prose.dialogue.word_frequency)
    combined_word_frequency.update(prose.narrative.word_frequency)
    assert(UNIQUE_WORD_COUNT == len(combined_word_frequency))
    assert(POV_WORD_COUNT == prose.dialogue.pov_word_count +
           prose.narrative.pov_word_count)
    assert(FIRST_PERSON_INDICATOR_COUNT == prose.dialogue.first_person_word_count +
           prose.narrative.first_person_word_count)
    assert(SECOND_PERSON_INDICATOR_COUNT == prose.dialogue.second_person_word_count +
           prose.narrative.second_person_word_count)
    assert(THIRD_PERSON_INDICATOR_COUNT == prose.dialogue.third_person_word_count +
           prose.narrative.third_person_word_count)


def test_readability_scores():
    assert(AUTOMATED_READABILITY_INDEX ==
           prose.readability_scores.automated_readability_index)
    assert(COLEMAN_LIAU_INDEX == prose.readability_scores.coleman_liau_index)
    assert(FLESCH_KINCAID_GRADE_LEVEL ==
           prose.readability_scores.flesch_kincaid_grade_level)
    assert(FLESCH_READING_EASE == prose.readability_scores.flesch_reading_ease)
    assert(GUNNING_FOG_INDEX == prose.readability_scores.gunning_fog_index)
    assert(LIX == prose.readability_scores.lix)
    assert(RIX == prose.readability_scores.rix)
    assert(SMOG == prose.readability_scores.smog)



# -*- coding: utf-8 -*-

import re

from prosegrinder.dictionary import Dictionary
from prosegrinder.fragment import Fragment


class Sentence(Fragment):

    RE_SENTENCE = re.compile("""
            # Match a sentence ending in punctuation or EOS.
            [^.!?…\\s]        # First char is non-punct, non-ws
            [^.!?…]*          # Greedily consume up to punctuation.
            (?:               # Group for unrolling the loop.
            [.!?…]            # (special) inner punctuation ok if
            (?!['\")]?\\s|$)  # not followed by ws or EOS.
            [^.!?…]*          # Greedily consume up to punctuation.
            )*                # Zero or more (special normal*)
            [.!?…]            # Ending punctuation.
            ['\")]?           # Optional closing quote.
            (?=\\s|$)
            """,
                             flags=re.MULTILINE | re.VERBOSE)

    RE_SMART_QUOTES = re.compile("[“”]")

    @staticmethod
    def parse_sentences(text, dictionary=Dictionary()):
        return [Sentence(sentence, dictionary) for sentence in re.findall(
            Sentence.RE_SENTENCE, text)]



# -*- coding: utf-8 -*-

from prosegrinder import Dictionary
from prosegrinder import Sentence

dictionary = Dictionary()
text = "This is a sentence of twelve words, and let's add some complexity."
sentence = Sentence(text, dictionary)

WORD_CHARACTER_COUNT = 53
WORD_COUNT = 12
PHONES = [
    'DH', 'IH1', 'S',
    'IH1', 'Z',
    'AH0',
    'S', 'EH1', 'N', 'T', 'AH0', 'N', 'S',
    'AH1', 'V',
    'T', 'W', 'EH1', 'L', 'V',
    'W', 'ER1', 'D', 'Z',
    'AH0', 'N', 'D',
    'L', 'EH1', 'T', 'S',
    'AE1', 'D',
    'S', 'AH1', 'M',
    'K', 'AH0', 'M', 'P', 'L', 'EH1', 'K', 'S', 'AH0', 'T', 'IY0',
]
SYLLABLE_COUNT = 16
COMPLEX_WORD_COUNT = 1

def test_sentence():
    assert(WORD_COUNT == sentence.word_count)
    assert(WORD_CHARACTER_COUNT == sentence.word_character_count)
    assert(len(PHONES) == sentence.phone_count)
    assert(SYLLABLE_COUNT == sentence.syllable_count)
    assert(COMPLEX_WORD_COUNT == sentence.complex_word_count)
